@@ -8,7 +8,7 @@
 
 //  xdebug_start_profiling();
 
-  if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__)).'/');
+  if(!defined('DOKU_INC')) define('DOKU_INC',dirname(__FILE__).'/');
   require_once(DOKU_INC.'inc/init.php');
   require_once(DOKU_INC.'inc/common.php');
   require_once(DOKU_INC.'inc/events.php');
