@@ -2,12 +2,10 @@
 /*************************************************************************************
  * html4strict.php
  * ---------------
- * Author: Nigel McNie (oracle.shinoda@gmail.com)
+ * Author: Nigel McNie (nigel@geshi.org)
  * Copyright: (c) 2004 Nigel McNie (http://qbnz.com/highlighter/)
- * Release Version: 1.0.7.7
- * CVS Revision Version: $Revision: 1.10 $
+ * Release Version: 1.0.7.21
  * Date Started: 2004/07/10
- * Last Modified: $Date: 2006/02/25 00:46:33 $
  *
  * HTML 4.01 strict language file for GeSHi.
  *
@@ -250,7 +248,8 @@ $language_data = array (
 		0 => false,
 		1 => false,
 		2 => true
-		)
+        ),
+    'TAB_WIDTH' => 4
 );
 
-?>
\ No newline at end of file
+?>
