@@ -2,8 +2,22 @@
 /*************************************************************************************
  * latex.php
  * -----
- * Original Author: efi
- * modified by Matthias Pospiech (mail@matthiaspospiech.de)
+ * Author: efi, Matthias Pospiech (mail@matthiaspospiech.de)
+ * Copyright: (c) 2006 efi, Matthias Pospiech (mail@matthiaspospiech.de), Nigel McNie (http://qbnz.com/highlighter)
+ * Release Version: 1.0.7.21
+ * Date Started: 2006/09/23
+ *
+ * LaTeX language file for GeSHi.
+ *
+ * CHANGES
+ * -------
+ * 2006/09/23 (1.0.0)
+ *  -  First Release
+ *
+ * TODO
+ * -------------------------
+ * *
+ *
  *************************************************************************************
  *
  *   This file is not yet part of GeSHi. (and is not compatible to the 1.1+ branch)
@@ -108,7 +122,7 @@ $language_data = array (
         3 => array(
             GESHI_SEARCH => "(\\{)(.*)(\\})",
             GESHI_REPLACE => '\2',
-            GESHI_MODIFIERS => '',
+            GESHI_MODIFIERS => 'U',
             GESHI_BEFORE => '\1',
             GESHI_AFTER => '\3'
             ),
@@ -116,7 +130,7 @@ $language_data = array (
         4 => array(
             GESHI_SEARCH => "(\[)(.+)(\])",
             GESHI_REPLACE => '\2',
-            GESHI_MODIFIERS => '',
+            GESHI_MODIFIERS => 'U',
             GESHI_BEFORE => '\1',
             GESHI_AFTER => '\3'
             ),
