@@ -2,12 +2,10 @@
 /*************************************************************************************
  * sdlbasic.php
  * ------------
- * Author: Roberto Rossi 
+ * Author: Roberto Rossi
  * Copyright: (c) 2005 Roberto Rossi (http://rsoftware.altervista.org)
- * Release Version: 1.0.7.7
- * CVS Revision Version: $Revision: 1.7 $
+ * Release Version: 1.0.7.21
  * Date Started: 2005/08/19
- * Date Modified: $Date: 2006/02/25 00:46:33 $
  *
  * sdlBasic (http://sdlbasic.sf.net) language file for GeSHi.
  *
@@ -36,7 +34,7 @@
  *
  ************************************************************************************/
  $language_data = array (
-	'LANG_NAME' => 'Visual Basic',
+	'LANG_NAME' => 'sdlBasic',
 	'COMMENT_SINGLE' => array(1 => "'", 2 => "rem", 3 => "!", 4 => "#"),
 	'COMMENT_MULTI' => array(),
 	'CASE_KEYWORDS' => GESHI_CAPS_NO_CHANGE,
@@ -104,7 +102,7 @@
     'acceptsock', 'isserverready', 'connectsock', 'connectionreadysock',
     'isclientready', 'losesock', 'peeksock', 'readsock', 'readbytesock',
     'readlinesock', 'writesock', 'writebytesock', 'writelinesock',
-    'getremoteip', 'getremoteport', 'getlocalip' 
+    'getremoteip', 'getremoteport', 'getlocalip'
       )
 		),
 	'SYMBOLS' => array(
