@@ -4,7 +4,7 @@
  * ----------
  * Author: Alan Juden (alan@judenware.org)
  * Copyright: (c) 2004 Alan Juden, Nigel McNie (http://qbnz.com/highlighter/)
- * Release Version: 1.0.7.19
+ * Release Version: 1.0.7.21
  * Date Started: 2004/07/10
  *
  * Smarty template language file for GeSHi.
@@ -160,7 +160,13 @@ $language_data = array (
 	),
 	'HIGHLIGHT_STRICT_BLOCK' => array(
 		0 => true
-		)
+	),
+	'PARSER_CONTROL' => array(
+        'KEYWORDS' => array(
+            'DISALLOWED_BEFORE' => "a-zA-Z0-9\$_\|\#;>|^",
+            'DISALLOWED_AFTER' => "a-zA-Z0-9_<\|%\\-&"
+        )
+    )
 );
 
 ?>
