@@ -6,9 +6,10 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
+if(!defined('DOKU_INC')) define('DOKU_INC',fullpath(dirname(__FILE__).'/../').'/');
 if(!defined('NL')) define('NL',"\n");
 require_once(DOKU_INC.'inc/parserutils.php');
+require_once(DOKU_INC.'inc/form.php');
 
 /**
  * Convenience function to quickly build a wikilink
@@ -42,7 +43,6 @@ function html_attbuild($attributes){
  * The loginform
  *
  * @author   Andreas Gohr <andi@splitbrain.org>
- * @triggers HTML_LOGINFORM_INJECTION
  */
 function html_login(){
   global $lang;
@@ -51,59 +51,32 @@ function html_login(){
   global $auth;
 
   print p_locale_xhtml('login');
-  ?>
-    <div class="centeralign">
-    <form action="<?php echo script()?>" accept-charset="<?php echo $lang['encoding']?>"
-          method="post" id="dw__login">
-      <fieldset>
-        <legend><?php echo $lang['btn_login']?></legend>
-        <input type="hidden" name="id" value="<?php echo $ID?>" />
-        <input type="hidden" name="do" value="login" />
-        <label class="block">
-          <span><?php echo $lang['user']?></span>
-          <input type="text" name="u" value="<?php echo formText($_REQUEST['u'])?>"
-                 class="edit" id="focus__this" />
-        </label><br />
-        <label class="block">
-          <span><?php echo $lang['pass']?></span>
-          <input type="password" name="p" class="edit" />
-        </label><br />
+  print '<div class="centeralign">'.NL;
+  $form = new Doku_Form('dw__login');
+  $form->startFieldset($lang['btn_login']);
+  $form->addHidden('id', $ID);
+  $form->addHidden('do', 'login');
+  $form->addElement(form_makeTextField('u', $_REQUEST['u'], $lang['user'], 'focus__this', 'block'));
+  $form->addElement(form_makePasswordField('p', $lang['pass'], '', 'block'));
+  $form->addElement(form_makeCheckboxField('r', '1', $lang['remember'], 'remember__me', 'simple'));
+  $form->addElement(form_makeButton('submit', '', $lang['btn_login']));
+  $form->endFieldset();
+  html_form('login', $form);
 
-        <?php //bad and dirty event insert hook
-        $evdata = array();
-        trigger_event('HTML_LOGINFORM_INJECTION', $evdata);
-        ?>
-
-        <label for="remember__me" class="simple">
-          <input type="checkbox" name="r" id="remember__me" value="1" />
-          <span><?php echo $lang['remember']?></span>
-        </label>
-        <input type="submit" value="<?php echo $lang['btn_login']?>" class="button" />
-      </fieldset>
-    </form>
-  <?php
-    if($auth->canDo('addUser') && actionOK('register')){
-      print '<p>';
-      print $lang['reghere'];
-      print ': <a href="'.wl($ID,'do=register').'" rel="nofollow" class="wikilink1">'.$lang['register'].'</a>';
-      print '</p>';
-    }
-
-    if ($auth->canDo('modPass') && actionOK('resendpwd')) {
-      print '<p>';
-      print $lang['pwdforget'];
-      print ': <a href="'.wl($ID,'do=resendpwd').'" rel="nofollow" class="wikilink1">'.$lang['btn_resendpwd'].'</a>';
-      print '</p>';
-    }
-  ?>
-    </div>
-  <?php
-/*
- FIXME provide new hook
-  if(@file_exists('includes/login.txt')){
-    print io_cacheParse('includes/login.txt');
+  if($auth && $auth->canDo('addUser') && actionOK('register')){
+    print '<p>';
+    print $lang['reghere'];
+    print ': <a href="'.wl($ID,'do=register').'" rel="nofollow" class="wikilink1">'.$lang['register'].'</a>';
+    print '</p>';
   }
-*/
+
+  if ($auth && $auth->canDo('modPass') && actionOK('resendpwd')) {
+    print '<p>';
+    print $lang['pwdforget'];
+    print ': <a href="'.wl($ID,'do=resendpwd').'" rel="nofollow" class="wikilink1">'.$lang['btn_resendpwd'].'</a>';
+    print '</p>';
+  }
+  print '</div>'.NL;
 }
 
 /**
@@ -190,7 +163,7 @@ function html_btn($name,$id,$akey,$params,$method='get',$tooltip=''){
     $params['id'] = $id;
   }
 
-  $ret .= '<form class="button" method="'.$method.'" action="'.$script.'"><div class="no">';
+  $ret .= '<form class="button btn_'.$name.'" method="'.$method.'" action="'.$script.'"><div class="no">';
 
   if(is_array($params)){
     reset($params);
@@ -227,6 +200,7 @@ function html_show($txt=''){
   global $ID;
   global $REV;
   global $HIGH;
+  global $INFO;
   //disable section editing for old revisions or in preview
   if($txt || $REV){
     $secedit = false;
@@ -236,18 +210,22 @@ function html_show($txt=''){
 
   if ($txt){
     //PreviewHeader
-    print '<br id="scroll__here" />';
-    print p_locale_xhtml('preview');
-    print '<div class="preview">';
-    print html_secedit(p_render('xhtml',p_get_instructions($txt),$info),$secedit);
-    print '<div class="clearer"></div>';
-    print '</div>';
+    echo '<br id="scroll__here" />';
+    echo p_locale_xhtml('preview');
+    echo '<div class="preview">';
+    $html = html_secedit(p_render('xhtml',p_get_instructions($txt),$info),$secedit);
+    if($INFO['prependTOC']) $html = tpl_toc(true).$html;
+    echo $html;
+    echo '<div class="clearer"></div>';
+    echo '</div>';
 
   }else{
     if ($REV) print p_locale_xhtml('showrev');
     $html = p_wiki_xhtml($ID,$REV,true);
     $html = html_secedit($html,$secedit);
-    print html_hilight($html,$HIGH);
+    if($INFO['prependTOC']) $html = tpl_toc(true).$html;
+    $html = html_hilight($html,$HIGH);
+    echo $html;
   }
 }
 
@@ -264,21 +242,18 @@ function html_draft(){
   $draft = unserialize(io_readFile($INFO['draft'],false));
   $text  = cleanText(con($draft['prefix'],$draft['text'],$draft['suffix'],true));
 
-  echo p_locale_xhtml('draft');
-  ?>
-  <form id="dw__editform" method="post" action="<?php echo script()?>"
-   accept-charset="<?php echo $lang['encoding']?>"><div class="no">
-    <input type="hidden" name="id"   value="<?php echo $ID?>" />
-    <input type="hidden" name="date" value="<?php echo $draft['date']?>" /></div>
-    <textarea name="wikitext" id="wiki__text" readonly="readonly" cols="80" rows="10" class="edit"><?php echo "\n".formText($text)?></textarea>
-
-    <div id="draft__status"><?php echo $lang['draftdate'].' '.date($conf['dformat'],filemtime($INFO['draft']))?></div>
-
-    <input class="button" type="submit" name="do[recover]" value="<?php echo $lang['btn_recover']?>" tabindex="1" />
-    <input class="button" type="submit" name="do[draftdel]" value="<?php echo $lang['btn_draftdel']?>" tabindex="2" />
-    <input class="button" type="submit" name="do[show]" value="<?php echo $lang['btn_cancel']?>" tabindex="3" />
-  </form>
-  <?php
+  print p_locale_xhtml('draft');
+  $form = new Doku_Form('dw__editform');
+  $form->addHidden('id', $ID);
+  $form->addHidden('date', $draft['date']);
+  $form->addElement(form_makeWikiText($text, array('readonly'=>'readonly')));
+  $form->addElement(form_makeOpenTag('div', array('id'=>'draft__status')));
+  $form->addElement($lang['draftdate'].' '. strftime($conf['dformat'],filemtime($INFO['draft'])));
+  $form->addElement(form_makeCloseTag('div'));
+  $form->addElement(form_makeButton('submit', 'recover', $lang['btn_recover'], array('tabindex'=>'1')));
+  $form->addElement(form_makeButton('submit', 'draftdel', $lang['btn_draftdel'], array('tabindex'=>'2')));
+  $form->addElement(form_makeButton('submit', 'show', $lang['btn_cancel'], array('tabindex'=>'3')));
+  html_form('draft', $form);
 }
 
 /**
@@ -287,13 +262,11 @@ function html_draft(){
  * @author Andreas Gohr <andi@splitbrain.org>
  * @author Harry Fuecks <hfuecks@gmail.com>
  */
-function html_hilight($html,$query){
-  //split at common delimiters
-  $queries = preg_split ('/[\s\'"\\\\`()\]\[?:!\.{};,#+*<>\\/]+/',$query,-1,PREG_SPLIT_NO_EMPTY);
-  foreach ($queries as $q){
-     $q = preg_quote($q,'/');
-     $html = preg_replace_callback("/((<[^>]*)|$q)/i",'html_hilight_callback',$html);
-  }
+function html_hilight($html,$phrases){
+  $regex = join('|',array_map('preg_quote_cb',array_filter((array) $phrases)));
+
+  if ($regex === '') return $html;
+  $html = preg_replace_callback("/((<[^>]*)|$regex)/ui",'html_hilight_callback',$html);
   return $html;
 }
 
@@ -357,7 +330,7 @@ function html_search(){
     print '<ul class="search_quickhits">';
     foreach($data as $id){
       print '<li> ';
-      print html_wikilink(':'.$id,$conf['useheading']?NULL:$id);
+      print html_wikilink(':'.$id,noNS($id));
       print '</li> ';
     }
     print '</ul> ';
@@ -368,15 +341,15 @@ function html_search(){
   flush();
 
   //do fulltext search
-  $data = ft_pageSearch($QUERY,$poswords);
+  $data = ft_pageSearch($QUERY,$regex);
   if(count($data)){
     $num = 1;
     foreach($data as $id => $cnt){
       print '<div class="search_result">';
-      print html_wikilink(':'.$id,$conf['useheading']?NULL:$id,$poswords);
+      print html_wikilink(':'.$id,$conf['useheading']?NULL:$id,$regex);
       print ': <span class="search_cnt">'.$cnt.' '.$lang['hits'].'</span><br />';
       if($num < 15){ // create snippets for the first number of matches only #FIXME add to conf ?
-        print '<div class="search_snippet">'.ft_snippet($id,$poswords).'</div>';
+        print '<div class="search_snippet">'.ft_snippet($id,$regex).'</div>';
       }
       print '</div>';
       flush();
@@ -405,7 +378,7 @@ function html_locked(){
   global $INFO;
 
   $locktime = filemtime(wikiLockFN($ID));
-  $expire = @date($conf['dformat'], $locktime + $conf['locktime'] );
+  $expire = @strftime($conf['dformat'], $locktime + $conf['locktime'] );
   $min    = round(($conf['locktime'] - (time() - $locktime) )/60);
 
   print p_locale_xhtml('locked');
@@ -441,13 +414,15 @@ function html_revisions($first=0){
     array_pop($revisions); // remove extra log entry
   }
 
-  $date = @date($conf['dformat'],$INFO['lastmod']);
+  $date = @strftime($conf['dformat'],$INFO['lastmod']);
 
   print p_locale_xhtml('revisions');
+  print '<form action="'.wl($ID).'" method="post" id="page__revisions">';
   print '<ul>';
   if($INFO['exists'] && $first==0){
     print (isset($INFO['meta']) && isset($INFO['meta']['last_change']) && $INFO['meta']['last_change']['type']===DOKU_CHANGE_TYPE_MINOR_EDIT) ? '<li class="minor">' : '<li>';
     print '<div class="li">';
+    print '<input type="checkbox" name="rev2[]" value="current" /> ';
 
     print $date;
 
@@ -456,9 +431,9 @@ function html_revisions($first=0){
     print '<a class="wikilink1" href="'.wl($ID).'">'.$ID.'</a> ';
 
     print ' &ndash; ';
-    print $INFO['sum'];
+    print htmlspecialchars($INFO['sum']);
     print ' <span class="user">';
-    print (empty($INFO['editor']))?('('.$lang['external_edit'].')'):$INFO['editor'];
+    print (empty($INFO['editor']))?('('.$lang['external_edit'].')'):htmlspecialchars($INFO['editor']);
     print '</span> ';
 
     print '('.$lang['current'].')';
@@ -467,14 +442,20 @@ function html_revisions($first=0){
   }
 
   foreach($revisions as $rev){
-    $date = date($conf['dformat'],$rev);
-    $info = getRevisionInfo($ID,$rev,true);
+    $date   = strftime($conf['dformat'],$rev);
+    $info   = getRevisionInfo($ID,$rev,true);
+    $exists = page_exists($ID,$rev);
 
     print ($info['type']===DOKU_CHANGE_TYPE_MINOR_EDIT) ? '<li class="minor">' : '<li>';
     print '<div class="li">';
+    if($exists){
+      print '<input type="checkbox" name="rev2[]" value="'.$rev.'" /> ';
+    }else{
+      print '<img src="'.DOKU_BASE.'lib/images/blank.gif" width="14" height="11" alt="" /> ';
+    }
     print $date;
 
-    if(@file_exists(wikiFN($ID,$rev))){
+    if($exists){
       print ' <a href="'.wl($ID,"rev=$rev,do=diff").'">';
       $p = array();
       $p['src']    = DOKU_BASE.'lib/images/diff.png';
@@ -496,7 +477,7 @@ function html_revisions($first=0){
     print htmlspecialchars($info['sum']);
     print ' <span class="user">';
     if($info['user']){
-      print $info['user'];
+      print htmlspecialchars($info['user']);
     }else{
       print $info['ip'];
     }
@@ -506,6 +487,8 @@ function html_revisions($first=0){
     print '</li>';
   }
   print '</ul>';
+  print '<input name="do[diff]" type="submit" value="'.$lang['diff2'].'" class="button" />';
+  print '</form>';
 
   print '<div class="pagenav">';
   $last = $first + $conf['recent'];
@@ -555,7 +538,7 @@ function html_recent($first=0){
   print '<ul>';
 
   foreach($recents as $recent){
-    $date = date($conf['dformat'],$recent['date']);
+    $date = strftime($conf['dformat'],$recent['date']);
     print ($recent['type']===DOKU_CHANGE_TYPE_MINOR_EDIT) ? '<li class="minor">' : '<li>';
     print '<div class="li">';
 
@@ -588,7 +571,7 @@ function html_recent($first=0){
 
     print ' <span class="user">';
     if($recent['user']){
-      print $recent['user'];
+      print htmlspecialchars($recent['user']);
     }else{
       print $recent['ip'];
     }
@@ -634,11 +617,14 @@ function html_index($ns){
   }
   $ns  = utf8_encodeFN(str_replace(':','/',$ns));
 
-  print p_locale_xhtml('index');
+  echo p_locale_xhtml('index');
+  echo '<div id="index__tree">';
 
   $data = array();
   search($data,$conf['datadir'],'search_index',array('ns' => $ns));
-  print html_buildlist($data,'idx','html_list_index','html_li_index');
+  echo html_buildlist($data,'idx','html_list_index','html_li_index');
+
+  echo '</div>';
 }
 
 /**
@@ -654,7 +640,7 @@ function html_list_index($item){
   $base = ':'.$item['id'];
   $base = substr($base,strrpos($base,':')+1);
   if($item['type']=='d'){
-    $ret .= '<a href="'.wl($ID,'idx='.$item['id']).'" class="idx_dir"><strong>';
+    $ret .= '<a href="'.wl($ID,'idx='.rawurlencode($item['id'])).'" class="idx_dir"><strong>';
     $ret .= $base;
     $ret .= '</strong></a>';
   }else{
@@ -754,23 +740,29 @@ function html_buildlist($data,$class,$func,$lifunc='html_li_default'){
  * display backlinks
  *
  * @author Andreas Gohr <andi@splitbrain.org>
+ * @author Michael Klier <chi@chimeric.de>
  */
 function html_backlinks(){
   require_once(DOKU_INC.'inc/fulltext.php');
   global $ID;
   global $conf;
+  global $lang;
 
   print p_locale_xhtml('backlinks');
 
   $data = ft_backlinks($ID);
 
-  print '<ul class="idx">';
-  foreach($data as $blink){
-    print '<li><div class="li">';
-    print html_wikilink(':'.$blink,$conf['useheading']?NULL:$blink);
-    print '</div></li>';
+  if(!empty($data)) {
+      print '<ul class="idx">';
+      foreach($data as $blink){
+        print '<li><div class="li">';
+        print html_wikilink(':'.$blink,$conf['useheading']?NULL:$blink);
+        print '</div></li>';
+      }
+      print '</ul>';
+  } else {
+      print '<div class="level1"><p>' . $lang['nothingfound'] . '</p></div>';
   }
-  print '</ul>';
 }
 
 /**
@@ -785,54 +777,93 @@ function html_diff($text='',$intro=true){
   global $lang;
   global $conf;
 
-  if($text){
-    $df  = new Diff(explode("\n",htmlspecialchars(rawWiki($ID,''))),
-                    explode("\n",htmlspecialchars(cleanText($text))));
-    $left  = '<a class="wikilink1" href="'.wl($ID).'">'.
-              $ID.' '.date($conf['dformat'],@filemtime(wikiFN($ID))).'</a>'.
-              $lang['current'];
-    $right = $lang['yours'];
-  }else{
-    //check if current revision exist
-    if(!@file_exists(wikiFN($ID))){
-      $revs = getRevisions($ID, 0, 2);
-      $rc = $revs[1];
+  // we're trying to be clever here, revisions to compare can be either
+  // given as rev and rev2 parameters, with rev2 being optional. Or in an
+  // array in rev2.
+  $rev1 = $REV;
+
+  if(is_array($_REQUEST['rev2'])){
+    $rev1 = (int) $_REQUEST['rev2'][0];
+    $rev2 = (int) $_REQUEST['rev2'][1];
+
+    if(!$rev1){
+        $rev1 = $rev2;
+        unset($rev2);
     }
-    if($REV){
-      $r = $REV;
-    }else{
-      if(empty($revs)){
-        //use last revision if none given
-        $revs = getRevisions($ID, 0, 1);
+  }else{
+    $rev2 = (int) $_REQUEST['rev2'];
+  }
+
+  if($text){                      // compare text to the most current revision
+    $l_rev   = '';
+    $l_text  = rawWiki($ID,'');
+    $l_head  = '<a class="wikilink1" href="'.wl($ID).'">'.
+               $ID.' '.strftime($conf['dformat'],@filemtime(wikiFN($ID))).'</a> '.
+               $lang['current'];
+
+    $r_rev   = '';
+    $r_text  = cleanText($text);
+    $r_head  = $lang['yours'];
+  }else{
+    if($rev1 && $rev2){            // two specific revisions wanted
+      // make sure order is correct (older on the left)
+      if($rev1 < $rev2){
+        $l_rev = $rev1;
+        $r_rev = $rev2;
+      }else{
+        $l_rev = $rev2;
+        $r_rev = $rev1;
       }
-      $r = $revs[0];
+    }elseif($rev1){                // single revision given, compare to current
+      $r_rev = '';
+      $l_rev = $rev1;
+    }else{                        // no revision was given, compare previous to current
+      $r_rev = '';
+      $revs = getRevisions($ID, 0, 1);
+      $l_rev = $revs[0];
     }
 
-    if($r){
-      $df  = new Diff(explode("\n",htmlspecialchars(rawWiki($ID,$r))),
-                      explode("\n",htmlspecialchars(rawWiki($ID,''))));
-      $left  = '<a class="wikilink1" href="'.wl($ID,"rev=$r").'">'.
-                $ID.' '.date($conf['dformat'],(isset($rc) ? $rc : $r)).'</a>';
+    // when both revisions are empty then the page was created just now
+    if(!$l_rev && !$r_rev){
+      $l_text = '';
     }else{
-      $df  = new Diff(array(''),
-                      explode("\n",htmlspecialchars(rawWiki($ID,''))));
-      $left  = '<a class="wikilink1" href="'.wl($ID).'">'.
-                $ID.'</a>';
+      $l_text = rawWiki($ID,$l_rev);
+    }
+    $r_text = rawWiki($ID,$r_rev);
+
+
+    if(!$l_rev){
+      $l_head = '&mdash;';
+    }else{
+      $l_head = '<a class="wikilink1" href="'.wl($ID,"rev=$l_rev").'">'.
+                $ID.' '.strftime($conf['dformat'],$l_rev).'</a>';
+    }
+
+    if($r_rev){
+      $r_head = '<a class="wikilink1" href="'.wl($ID,"rev=$r_rev").'">'.
+                $ID.' '.strftime($conf['dformat'],$r_rev).'</a>';
+    }elseif($_rev = @filemtime(wikiFN($ID))){
+      $r_head  = '<a class="wikilink1" href="'.wl($ID).'">'.
+               $ID.' '.strftime($conf['dformat'],$_rev).'</a> '.
+               $lang['current'];
+    }else{
+      $r_head = '&mdash; '.$lang['current'];
     }
-    $right = '<a class="wikilink1" href="'.wl($ID).'">'.
-              $ID.' '.date($conf['dformat'],(isset($rc) ? $r : @filemtime(wikiFN($ID)))).'</a> '.
-              $lang['current'];
   }
+
+  $df = new Diff(explode("\n",htmlspecialchars($l_text)),
+                 explode("\n",htmlspecialchars($r_text)));
+
   $tdf = new TableDiffFormatter();
   if($intro) print p_locale_xhtml('diff');
   ?>
     <table class="diff">
       <tr>
         <th colspan="2">
-          <?php echo $left?>
+          <?php echo $l_head?>
         </th>
         <th colspan="2">
-          <?php echo $right?>
+          <?php echo $r_head?>
         </th>
       </tr>
       <?php echo $tdf->format($df)?>
@@ -850,19 +881,14 @@ function html_conflict($text,$summary){
   global $lang;
 
   print p_locale_xhtml('conflict');
-  ?>
-  <form id="dw__editform" method="post" action="<?php echo script()?>" accept-charset="<?php echo $lang['encoding']?>">
-  <div class="centeralign">
-    <input type="hidden" name="id" value="<?php echo $ID?>" />
-    <input type="hidden" name="wikitext" value="<?php echo formText($text)?>" />
-    <input type="hidden" name="summary" value="<?php echo formText($summary)?>" />
-
-    <input class="button" type="submit" name="do[save]" value="<?php echo $lang['btn_save']?>" accesskey="s" title="<?php echo $lang['btn_save']?> [ALT+S]" />
-    <input class="button" type="submit" name="do[cancel]" value="<?php echo $lang['btn_cancel']?>" />
-  </div>
-  </form>
-  <br /><br /><br /><br />
-  <?php
+  $form = new Doku_Form('dw__editform');
+  $form->addHidden('id', $ID);
+  $form->addHidden('wikitext', $text);
+  $form->addHidden('summary', $summary);
+  $form->addElement(form_makeButton('submit', 'save', $lang['btn_save'], array('accesskey'=>'s')));
+  $form->addElement(form_makeButton('submit', 'cancel', $lang['btn_cancel']));
+  html_form('conflict', $form);
+  print '<br /><br /><br /><br />'.NL;
 }
 
 /**
@@ -886,7 +912,6 @@ function html_msgarea(){
  * Prints the registration form
  *
  * @author Andreas Gohr <andi@splitbrain.org>
- * @triggers HTML_REGISTERFORM_INJECTION
  */
 function html_register(){
   global $lang;
@@ -894,51 +919,23 @@ function html_register(){
   global $ID;
 
   print p_locale_xhtml('register');
-?>
-  <div class="centeralign">
-  <form id="dw__register" method="post" action="<?php echo wl($ID)?>" accept-charset="<?php echo $lang['encoding']?>">
-  <fieldset>
-    <input type="hidden" name="do" value="register" />
-    <input type="hidden" name="save" value="1" />
+  print '<div class="centeralign">'.NL;
+  $form = new Doku_Form('dw__register', wl($ID));
+  $form->startFieldset($lang['register']);
+  $form->addHidden('do', 'register');
+  $form->addHidden('save', '1');
+  $form->addElement(form_makeTextField('login', $_POST['login'], $lang['user'], null, 'block', array('size'=>'50')));
+  if (!$conf['autopasswd']) {
+    $form->addElement(form_makePasswordField('pass', $lang['pass'], '', 'block', array('size'=>'50')));
+    $form->addElement(form_makePasswordField('passchk', $lang['passchk'], '', 'block', array('size'=>'50')));
+  }
+  $form->addElement(form_makeTextField('fullname', $_POST['fullname'], $lang['fullname'], '', 'block', array('size'=>'50')));
+  $form->addElement(form_makeTextField('email', $_POST['email'], $lang['email'], '', 'block', array('size'=>'50')));
+  $form->addElement(form_makeButton('submit', '', $lang['register']));
+  $form->endFieldset();
+  html_form('register', $form);
 
-    <legend><?php echo $lang['register']?></legend>
-    <label class="block">
-      <?php echo $lang['user']?>
-      <input type="text" name="login" class="edit" size="50" value="<?php echo formText($_POST['login'])?>" />
-    </label><br />
-
-    <?php
-      if (!$conf['autopasswd']) {
-    ?>
-      <label class="block">
-        <?php echo $lang['pass']?>
-        <input type="password" name="pass" class="edit" size="50" />
-      </label><br />
-      <label class="block">
-        <?php echo $lang['passchk']?>
-        <input type="password" name="passchk" class="edit" size="50" />
-      </label><br />
-    <?php
-      }
-    ?>
-
-    <label class="block">
-      <?php echo $lang['fullname']?>
-      <input type="text" name="fullname" class="edit" size="50" value="<?php echo formText($_POST['fullname'])?>" />
-    </label><br />
-    <label class="block">
-      <?php echo $lang['email']?>
-      <input type="text" name="email" class="edit" size="50" value="<?php echo formText($_POST['email'])?>" />
-    </label><br />
-    <?php //bad and dirty event insert hook
-    $evdata = array();
-    trigger_event('HTML_REGISTERFORM_INJECTION', $evdata);
-    ?>
-    <input type="submit" class="button" value="<?php echo $lang['register']?>" />
-  </fieldset>
-  </form>
-  </div>
-<?php
+  print '</div>'.NL;
 }
 
 /**
@@ -958,55 +955,32 @@ function html_updateprofile(){
 
   if (empty($_POST['fullname'])) $_POST['fullname'] = $INFO['userinfo']['name'];
   if (empty($_POST['email'])) $_POST['email'] = $INFO['userinfo']['mail'];
-?>
-  <div class="centeralign">
-  <form id="dw__register" method="post" action="<?php echo wl($ID)?>" accept-charset="<?php echo $lang['encoding']?>">
-  <fieldset style="width: 80%;">
-    <input type="hidden" name="do" value="profile" />
-    <input type="hidden" name="save" value="1" />
-
-    <legend><?php echo $lang['profile']?></legend>
-    <label class="block">
-      <?php echo $lang['user']?>
-      <input type="text" name="fullname" disabled="disabled" class="edit" size="50" value="<?php echo formText($_SERVER['REMOTE_USER'])?>" />
-    </label><br />
-    <label class="block">
-      <?php echo $lang['fullname']?>
-      <input type="text" name="fullname" <?php if(!$auth->canDo('modName')) echo 'disabled="disabled"'?> class="edit" size="50" value="<?php echo formText($_POST['fullname'])?>" />
-    </label><br />
-    <label class="block">
-      <?php echo $lang['email']?>
-      <input type="text" name="email" <?php if(!$auth->canDo('modName')) echo 'disabled="disabled"'?> class="edit" size="50" value="<?php echo formText($_POST['email'])?>" />
-    </label><br /><br />
-
-    <?php if($auth->canDo('modPass')) { ?>
-    <label class="block">
-      <?php echo $lang['newpass']?>
-      <input type="password" name="newpass" class="edit" size="50" />
-    </label><br />
-    <label class="block">
-      <?php echo $lang['passchk']?>
-      <input type="password" name="passchk" class="edit" size="50" />
-    </label><br />
-    <?php } ?>
-
-    <?php if ($conf['profileconfirm']) { ?>
-      <br />
-      <label class="block">
-      <?php echo $lang['oldpass']?>
-      <input type="password" name="oldpass" class="edit" size="50" />
-    </label><br />
-    <?php } ?>
-    <?php //bad and dirty event insert hook
-    $evdata = array();
-    trigger_event('HTML_PROFILEFORM_INJECTION', $evdata);
-    ?>
-    <input type="submit" class="button" value="<?php echo $lang['btn_save']?>" />
-    <input type="reset" class="button" value="<?php echo $lang['btn_reset']?>" />
-  </fieldset>
-  </form>
-  </div>
-<?php
+  print '<div class="centeralign">'.NL;
+  $form = new Doku_Form('dw__register', wl($ID));
+  $form->startFieldset($lang['profile']);
+  $form->addHidden('do', 'profile');
+  $form->addHidden('save', '1');
+  $form->addElement(form_makeTextField('fullname', $_SERVER['REMOTE_USER'], $lang['user'], '', 'block', array('size'=>'50', 'disabled'=>'disabled')));
+  $attr = array('size'=>'50');
+  if (!$auth->canDo('modName')) $attr['disabled'] = 'disabled';
+  $form->addElement(form_makeTextField('fullname', $_POST['fullname'], $lang['fullname'], '', 'block', $attr));
+  $attr = array('size'=>'50');
+  if (!$auth->canDo('modMail')) $attr['disabled'] = 'disabled';
+  $form->addElement(form_makeTextField('email', $_POST['email'], $lang['email'], '', 'block', $attr));
+  $form->addElement(form_makeTag('br'));
+  if ($auth->canDo('modPass')) {
+    $form->addElement(form_makePasswordField('newpass', $lang['newpass'], '', 'block', array('size'=>'50')));
+    $form->addElement(form_makePasswordField('passchk', $lang['passchk'], '', 'block', array('size'=>'50')));
+  }
+  if ($conf['profileconfirm']) {
+    $form->addElement(form_makeTag('br'));
+    $form->addElement(form_makePasswordField('oldpass', $lang['oldpass'], '', 'block', array('size'=>'50')));
+  }
+  $form->addElement(form_makeButton('submit', '', $lang['btn_save']));
+  $form->addElement(form_makeButton('reset', '', $lang['btn_reset']));
+  $form->endFieldset();
+  html_form('updateprofile', $form);
+  print '</div>'.NL;
 }
 
 /**
@@ -1047,20 +1021,31 @@ function html_edit($text=null,$include='edit'){ //FIXME: include needed?
       }else{
         $text = rawWiki($ID,$REV);
       }
+      $check = md5($text);
+      $mod = false;
     }else{
       //try to load a pagetemplate
       $data = array($ID);
       $text = trigger_event('HTML_PAGE_FROMTEMPLATE',$data,'pageTemplate',true);
+      $check = md5('');
+      $mod = $text!=='';
     }
   }else{
     $pr = true; //preview mode
+    if (isset($_REQUEST['changecheck'])) {
+      $check = $_REQUEST['changecheck'];
+      $mod = md5($text)!==$check;
+    } else {
+      // Why? Assume default text is unmodified.
+      $check = md5($text);
+      $mod = false;
+    }
   }
 
   $wr = $INFO['writable'];
   if($wr){
     if ($REV) print p_locale_xhtml('editrev');
     print p_locale_xhtml($include);
-    $ro=false;
   }else{
     // check pseudo action 'source'
     if(!actionOK('source')){
@@ -1068,7 +1053,6 @@ function html_edit($text=null,$include='edit'){ //FIXME: include needed?
       return;
     }
     print p_locale_xhtml('read');
-    $ro='readonly="readonly"';
   }
   if(!$DATE) $DATE = $INFO['lastmod'];
 
@@ -1077,57 +1061,49 @@ function html_edit($text=null,$include='edit'){ //FIXME: include needed?
   <div style="width:99%;">
 
    <div class="toolbar">
-      <div id="draft__status"><?php if(!empty($INFO['draft'])) echo $lang['draftdate'].' '.date($conf['dformat']);?></div>
-      <div id="tool__bar"><?php if(!$ro){?><a href="<?php echo DOKU_BASE?>lib/exe/mediamanager.php?ns=<?php echo $INFO['namespace']?>"
+      <div id="draft__status"><?php if(!empty($INFO['draft'])) echo $lang['draftdate'].' '.strftime($conf['dformat']);?></div>
+      <div id="tool__bar"><?php if($wr){?><a href="<?php echo DOKU_BASE?>lib/exe/mediamanager.php?ns=<?php echo $INFO['namespace']?>"
       target="_blank"><?php echo $lang['mediaselect'] ?></a><?php }?></div>
 
       <?php if($wr){?>
       <script type="text/javascript" charset="utf-8"><!--//--><![CDATA[//><!--
         <?php /* sets changed to true when previewed */?>
-        textChanged = <?php ($pr) ? print 'true' : print 'false' ?>;
+        textChanged = <?php ($mod) ? print 'true' : print 'false' ?>;
       //--><!]]></script>
       <span id="spell__action"></span>
       <div id="spell__suggest"></div>
       <?php } ?>
    </div>
    <div id="spell__result"></div>
-
-
-   <form id="dw__editform" method="post" action="<?php echo script()?>" accept-charset="<?php echo $lang['encoding']?>"><div class="no">
-      <input type="hidden" name="id"   value="<?php echo $ID?>" />
-      <input type="hidden" name="rev"  value="<?php echo $REV?>" />
-      <input type="hidden" name="date" value="<?php echo $DATE?>" />
-      <input type="hidden" name="prefix" value="<?php echo formText($PRE)?>" />
-      <input type="hidden" name="suffix" value="<?php echo formText($SUF)?>" />
-    </div>
-
-    <textarea name="wikitext" id="wiki__text" <?php echo $ro?> cols="80" rows="10" class="edit" tabindex="1"><?php echo "\n".formText($text)?></textarea>
-
-    <?php //bad and dirty event insert hook
-    $evdata = array('writable' => $wr);
-    trigger_event('HTML_EDITFORM_INJECTION', $evdata);
-    ?>
-
-    <div id="wiki__editbar">
-      <div id="size__ctl"></div>
-      <?php if($wr){?>
-         <div class="editButtons">
-            <input class="button" id="edbtn__save" type="submit" name="do[save]" value="<?php echo $lang['btn_save']?>" accesskey="s" title="<?php echo $lang['btn_save']?> [ALT+S]" tabindex="4" />
-            <input class="button" id="edbtn__preview" type="submit" name="do[preview]" value="<?php echo $lang['btn_preview']?>" accesskey="p" title="<?php echo $lang['btn_preview']?> [ALT+P]" tabindex="5" />
-            <input class="button" type="submit" name="do[draftdel]" value="<?php echo $lang['btn_cancel']?>" tabindex="6" />
-         </div>
-      <?php } ?>
-      <?php if($wr){ ?>
-        <div class="summary">
-           <label for="edit__summary" class="nowrap"><?php echo $lang['summary']?>:</label>
-           <input type="text" class="edit" name="summary" id="edit__summary" size="50" value="<?php echo formText($SUM)?>" tabindex="2" />
-           <?php html_minoredit()?>
-        </div>
-      <?php }?>
-    </div>
-  </form>
-  </div>
 <?php
+  $form = new Doku_Form('dw__editform');
+  $form->addHidden('id', $ID);
+  $form->addHidden('rev', $REV);
+  $form->addHidden('date', $DATE);
+  $form->addHidden('prefix', $PRE);
+  $form->addHidden('suffix', $SUF);
+  $form->addHidden('changecheck', $check);
+  $attr = array('tabindex'=>'1');
+  if (!$wr) $attr['readonly'] = 'readonly';
+  $form->addElement(form_makeWikiText($text, $attr));
+  $form->addElement(form_makeOpenTag('div', array('id'=>'wiki__editbar')));
+  $form->addElement(form_makeOpenTag('div', array('id'=>'size__ctl')));
+  $form->addElement(form_makeCloseTag('div'));
+  if ($wr) {
+    $form->addElement(form_makeOpenTag('div', array('class'=>'editButtons')));
+    $form->addElement(form_makeButton('submit', 'save', $lang['btn_save'], array('id'=>'edbtn__save', 'accesskey'=>'s', 'tabindex'=>'4')));
+    $form->addElement(form_makeButton('submit', 'preview', $lang['btn_preview'], array('id'=>'edbtn__preview', 'accesskey'=>'p', 'tabindex'=>'5')));
+    $form->addElement(form_makeButton('submit', 'draftdel', $lang['btn_cancel'], array('tabindex'=>'6')));
+    $form->addElement(form_makeCloseTag('div'));
+    $form->addElement(form_makeOpenTag('div', array('class'=>'summary')));
+    $form->addElement(form_makeTextField('summary', $SUM, $lang['summary'], 'edit__summary', 'nowrap', array('size'=>'50', 'tabindex'=>'2')));
+    $elem = html_minoredit();
+    if ($elem) $form->addElement($elem);
+    $form->addElement(form_makeCloseTag('div'));
+  }
+  $form->addElement(form_makeCloseTag('div'));
+  html_form('edit', $form);
+  print '</div>'.NL;
 }
 
 /**
@@ -1140,24 +1116,13 @@ function html_minoredit(){
   global $lang;
   // minor edits are for logged in users only
   if(!$conf['useacl'] || !$_SERVER['REMOTE_USER']){
-    return;
+    return false;
   }
 
   $p = array();
-  $p['name']     = 'minor';
-  $p['type']     = 'checkbox';
-  $p['id']       = 'minoredit';
   $p['tabindex'] = 3;
-  $p['value']    = '1';
   if(!empty($_REQUEST['minor'])) $p['checked']='checked';
-  $att = buildAttributes($p);
-
-  print '<span class="nowrap">';
-  print "<input $att />";
-  print '<label for="minoredit">';
-  print $lang['minoredit'];
-  print '</label>';
-  print '</span>';
+  return form_makeCheckboxField('minor', '1', $lang['minoredit'], 'minoredit', 'nowrap', $p);
 }
 
 /**
@@ -1292,23 +1257,90 @@ function html_resendpwd() {
   global $ID;
 
   print p_locale_xhtml('resendpwd');
-?>
-  <div class="centeralign">
-  <form id="dw__resendpwd" action="<?php echo wl($ID)?>" accept-charset="<?php echo $lang['encoding']?>" method="post">
-    <fieldset>
-      <br />
-      <legend><?php echo $lang['resendpwd']?></legend>
-      <input type="hidden" name="do" value="resendpwd" />
-      <input type="hidden" name="save" value="1" />
-      <label class="block">
-        <span><?php echo $lang['user']?></span>
-        <input type="text" name="login" value="<?php echo formText($_POST['login'])?>" class="edit" /><br /><br />
-      </label><br />
-      <input type="submit" value="<?php echo $lang['btn_resendpwd']?>" class="button" />
-    </fieldset>
-  </form>
-  </div>
-<?php
+  print '<div class="centeralign">'.NL;
+  $form = new Doku_Form('dw__resendpwd', wl($ID));
+  $form->startFieldset($lang['resendpwd']);
+  $form->addHidden('do', 'resendpwd');
+  $form->addHidden('save', '1');
+  $form->addElement(form_makeTag('br'));
+  $form->addElement(form_makeTextField('login', $_POST['login'], $lang['user'], '', 'block'));
+  $form->addElement(form_makeTag('br'));
+  $form->addElement(form_makeTag('br'));
+  $form->addElement(form_makeButton('submit', '', $lang['btn_resendpwd']));
+  $form->endFieldset();
+  html_form('resendpwd', $form);
+  print '</div>'.NL;
+}
+
+/**
+ * Return the TOC rendered to XHTML
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function html_TOC($toc){
+    if(!count($toc)) return '';
+    global $lang;
+    $out  = '<!-- TOC START -->'.DOKU_LF;
+    $out .= '<div class="toc">'.DOKU_LF;
+    $out .= '<div class="tocheader toctoggle" id="toc__header">';
+    $out .= $lang['toc'];
+    $out .= '</div>'.DOKU_LF;
+    $out .= '<div id="toc__inside">'.DOKU_LF;
+    $out .= html_buildlist($toc,'toc','html_list_toc');
+    $out .= '</div>'.DOKU_LF.'</div>'.DOKU_LF;
+    $out .= '<!-- TOC END -->'.DOKU_LF;
+    return $out;                                                                                                }
+
+/**
+ * Callback for html_buildlist
+ */
+function html_list_toc($item){
+    if($item['hid']){
+        $link = '#'.$item['hid'];
+    }else{
+        $link = $item['link'];
+    }
+
+    return '<span class="li"><a href="'.$link.'" class="toc">'.
+           hsc($item['title']).'</a></span>';
+}
+
+/**
+ * Helper function to build TOC items
+ *
+ * Returns an array ready to be added to a TOC array
+ *
+ * @param string $link  - where to link (if $hash set to '#' it's a local anchor)
+ * @param string $text  - what to display in the TOC
+ * @param int    $level - nesting level
+ * @param string $hash  - is prepended to the given $link, set blank if you want full links
+ */
+function html_mktocitem($link, $text, $level, $hash='#'){
+    global $conf;
+    return  array( 'link'  => $hash.$link,
+                   'title' => $text,
+                   'type'  => 'ul',
+                   'level' => $level);
+}
+
+/**
+ * Output a Doku_Form object.
+ * Triggers an event with the form name: HTML_{$name}FORM_OUTPUT
+ *
+ * @author Tom N Harris <tnharris@whoopdedo.org>
+ */
+function html_form($name, &$form) {
+  // Safety check in case the caller forgets.
+  $form->endFieldset();
+  trigger_event('HTML_'.strtoupper($name).'FORM_OUTPUT', $form, 'html_form_output', false);
+}
+
+/**
+ * Form print function.
+ * Just calls printForm() on the data object.
+ */
+function html_form_output($data) {
+  $data->printForm();
 }
 
 //Setup VIM: ex: et ts=2 enc=utf-8 :
