@@ -6,7 +6,7 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
+if(!defined('DOKU_INC')) define('DOKU_INC',fullpath(dirname(__FILE__).'/../').'/');
 if(!defined('NL')) define('NL',"\n");
 
 require_once(DOKU_INC.'inc/html.php');
@@ -27,13 +27,10 @@ function media_filesinuse($data,$id){
     echo '<p>'.hsc($lang['ref_inuse']).'</p>';
 
     $hidden=0; //count of hits without read permission
-    usort($data,'sort_search_fulltext');
     foreach($data as $row){
-        if(auth_quickaclcheck($row['id']) >= AUTH_READ){
+        if(auth_quickaclcheck($row) >= AUTH_READ && isVisiblePage($row)){
             echo '<div class="search_result">';
-            echo '<span class="mediaref_ref">'.$row['id'].'</span>';
-            echo ': <span class="search_cnt">'.$row['count'].' '.$lang['hits'].'</span><br />';
-            echo '<div class="search_snippet">'.$row['snippet'].'</div>';
+            echo '<span class="mediaref_ref">'.hsc($row).'</span>';
             echo '</div>';
         }else
         $hidden++;
@@ -50,7 +47,9 @@ function media_filesinuse($data,$id){
  */
 function media_metasave($id,$auth,$data){
     if($auth < AUTH_UPLOAD) return false;
+    if(!checkSecurityToken()) return false;
     global $lang;
+    global $conf;
     $src = mediaFN($id);
 
     $meta = new JpegMeta($src);
@@ -66,6 +65,7 @@ function media_metasave($id,$auth,$data){
     }
 
     if($meta->save()){
+        if($conf['fperm']) chmod($src, $conf['fperm']);
         msg($lang['metasaveok'],1);
         return $id;
     }else{
@@ -97,11 +97,13 @@ function media_metaform($id,$auth){
     // output
     echo '<h1>'.hsc(noNS($id)).'</h1>'.NL;
     echo '<form action="'.DOKU_BASE.'lib/exe/mediamanager.php" accept-charset="utf-8" method="post" class="meta">'.NL;
+    formSecurityToken();
     foreach($fields as $key => $field){
         // get current value
         $tags = array($field[0]);
         if(is_array($field[3])) $tags = array_merge($tags,$field[3]);
         $value = tpl_img_getTag($tags,'',$src);
+        $value = cleanText($value);
 
         // prepare attributes
         $p = array();
@@ -137,6 +139,7 @@ function media_metaform($id,$auth){
     echo '</form>'.NL;
 }
 
+
 /**
  * Handles media file deletions
  *
@@ -147,19 +150,28 @@ function media_metaform($id,$auth){
  */
 function media_delete($id,$auth){
     if($auth < AUTH_DELETE) return false;
+    if(!checkSecurityToken()) return false;
     global $conf;
     global $lang;
 
+    // check for references if needed
     $mediareferences = array();
     if($conf['refcheck']){
-        search($mediareferences,$conf['datadir'],'search_reference',array('query' => $id));
+        require_once(DOKU_INC.'inc/fulltext.php');
+        $mediareferences = ft_mediause($id,$conf['refshow']);
     }
 
     if(!count($mediareferences)){
         $file = mediaFN($id);
         if(@unlink($file)){
             msg(str_replace('%s',noNS($id),$lang['deletesucc']),1);
-            io_sweepNS($id,'mediadir');
+            $del = io_sweepNS($id,'mediadir');
+            if($del){
+                // current namespace was removed. redirecting to root ns passing msg along
+                header('Location: '.DOKU_URL.'lib/exe/mediamanager.php?msg1='.
+                        rawurlencode(str_replace('%s',noNS($id),$lang['deletesucc'])));
+                exit;
+            }
             return true;
         }
         //something went wrong
@@ -176,11 +188,24 @@ function media_delete($id,$auth){
 /**
  * Handles media file uploads
  *
+ * This generates an action event and delegates to _media_upload_action().
+ * Action plugins are allowed to pre/postprocess the uploaded file. 
+ * (The triggered event is preventable.)
+ *
+ * Event data:
+ * $data[0]     fn_tmp: the temporary file name (read from $_FILES)
+ * $data[1]     fn: the file name of the uploaded file
+ * $data[2]     id: the future directory id of the uploaded file
+ * $data[3]     imime: the mimetype of the uploaded file
+ *
+ * @triggers MEDIA_UPLOAD_FINISH
  * @author Andreas Gohr <andi@splitbrain.org>
+ * @author Michael Klier <chi@chimeric.de>
  * @return mixed false on error, id of the new file on success
  */
 function media_upload($ns,$auth){
     if($auth < AUTH_UPLOAD) return false;
+    if(!checkSecurityToken()) return false;
     require_once(DOKU_INC.'inc/confutils.php');
     global $lang;
     global $conf;
@@ -231,25 +256,60 @@ function media_upload($ns,$auth){
             return false;
         }
 
-        // prepare directory
-        io_createNamespace($id, 'media');
-        if(move_uploaded_file($file['tmp_name'], $fn)) {
-            // Set the correct permission here.
-            // Always chmod media because they may be saved with different permissions than expected from the php umask.
-            // (Should normally chmod to $conf['fperm'] only if $conf['fperm'] is set.)
-            chmod($fn, $conf['fmode']);
-            msg($lang['uploadsucc'],1);
-            media_notify($id,$fn,$imime);
-            return $id;
-        }else{
-            msg($lang['uploadfail'],-1);
-        }
+        // prepare event data
+        $data[0] = $file['tmp_name'];
+        $data[1] = $fn;
+        $data[2] = $id;
+        $data[3] = $imime;
+
+        // trigger event
+        return trigger_event('MEDIA_UPLOAD_FINISH', $data, '_media_upload_action', true);
+
     }else{
         msg($lang['uploadwrong'],-1);
     }
     return false;
 }
 
+/**
+ * Callback adapter for media_upload_finish()
+ * @author Michael Klier <chi@chimeric.de>
+ */
+function _media_upload_action($data) {
+    // fixme do further sanity tests of given data?
+    if(is_array($data) && count($data)===4) {
+        return media_upload_finish($data[0], $data[1], $data[2], $data[3]);
+    } else {
+        return false; //callback error
+    }
+}
+
+/**
+ * Saves an uploaded media file
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ * @author Michael Klier <chi@chimeric.de>
+ */
+function media_upload_finish($fn_tmp, $fn, $id, $imime) {
+    global $conf;
+    global $lang;
+
+    // prepare directory
+    io_createNamespace($id, 'media');
+
+    if(move_uploaded_file($fn_tmp, $fn)) {
+        // Set the correct permission here.
+        // Always chmod media because they may be saved with different permissions than expected from the php umask.
+        // (Should normally chmod to $conf['fperm'] only if $conf['fperm'] is set.)
+        chmod($fn, $conf['fmode']);
+        msg($lang['uploadsucc'],1);
+        media_notify($id,$fn,$imime);
+        return $id;
+    }else{
+        msg($lang['uploadfail'],-1);
+    }
+}
+
 /**
  * This function checks if the uploaded content is really what the
  * mimetype says it is. We also do spam checking for text types here.
@@ -304,7 +364,7 @@ function media_notify($id,$file,$mime){
     if(empty($conf['notify'])) return; //notify enabled?
 
     $text = rawLocale('uploadmail');
-    $text = str_replace('@DATE@',date($conf['dformat']),$text);
+    $text = str_replace('@DATE@',strftime($conf['dformat']),$text);
     $text = str_replace('@BROWSER@',$_SERVER['HTTP_USER_AGENT'],$text);
     $text = str_replace('@IPADDRESS@',$_SERVER['REMOTE_ADDR'],$text);
     $text = str_replace('@HOSTNAME@',gethostbyaddr($_SERVER['REMOTE_ADDR']),$text);
@@ -347,7 +407,7 @@ function media_filelist($ns,$auth=null,$jump=''){
 
     $dir = utf8_encodeFN(str_replace(':','/',$ns));
     $data = array();
-    search($data,$conf['mediadir'],'search_media',array(),$dir);
+    search($data,$conf['mediadir'],'search_media',array('showmsg'=>true),$dir);
 
     if(!count($data)){
         echo '<div class="nothing">'.$lang['nothingfound'].'</div>'.NL;
@@ -382,7 +442,8 @@ function media_fileactions($item,$auth){
         $ask  = addslashes($lang['del_confirm']).'\\n';
         $ask .= addslashes($item['id']);
 
-        echo ' <a href="'.DOKU_BASE.'lib/exe/mediamanager.php?delete='.rawurlencode($item['id']).'" '.
+        echo ' <a href="'.DOKU_BASE.'lib/exe/mediamanager.php?delete='.rawurlencode($item['id']).
+             '&amp;sectok='.getSecurityToken().'" '.
              'onclick="return confirm(\''.$ask.'\')" onkeypress="return confirm(\''.$ask.'\')">'.
              '<img src="'.DOKU_BASE.'lib/images/trash.png" alt="'.$lang['btn_delete'].'" '.
              'title="'.$lang['btn_delete'].'" class="btn" /></a>';
@@ -433,7 +494,7 @@ function media_printfile($item,$auth,$jump){
         $info .= (int) $item['meta']->getField('File.Height');
         $info .= ' ';
     }
-    $info .= '<i>'.date($conf['dformat'],$item['mtime']).'</i>';
+    $info .= '<i>'.strftime($conf['dformat'],$item['mtime']).'</i>';
     $info .= ' ';
     $info .= filesize_h($item['size']);
 
@@ -507,41 +568,27 @@ function media_uploadform($ns, $auth){
 
     if($auth < AUTH_UPLOAD) return; //fixme print info on missing permissions?
 
-    ?>
-    <div class="upload"><?php echo $lang['mediaupload']?></div>
-    <form action="<?php echo DOKU_BASE?>lib/exe/mediamanager.php"
-          method="post" enctype="multipart/form-data" class="upload">
-      <fieldset>
-        <legend class="hidden"><?php echo $lang['btn_upload']?></legend>
-        <input type="hidden" name="ns" value="<?php echo hsc($ns)?>" />
+    print '<div class="upload">' . $lang['mediaupload'] . '</div>';
+    $form = new Doku_Form('dw__upload', DOKU_BASE.'lib/exe/mediamanager.php', false, 'multipart/form-data');
+    $form->addElement(formSecurityToken());
+    $form->addHidden('ns', hsc($ns));
+    $form->addElement(form_makeOpenTag('p'));
+    $form->addElement(form_makeFileField('upload', $lang['txt_upload'].':', 'upload__file'));
+    $form->addElement(form_makeCloseTag('p'));
+    $form->addElement(form_makeOpenTag('p'));
+    $form->addElement(form_makeTextField('id', '', $lang['txt_filename'].':', 'upload__name'));
+    $form->addElement(form_makeButton('submit', '', $lang['btn_upload']));
+    $form->addElement(form_makeCloseTag('p'));
 
-        <p>
-          <label for="upload__file"><?php echo $lang['txt_upload']?>:</label>
-          <input type="file" name="upload" class="edit" id="upload__file" />
-        </p>
+    if($auth >= AUTH_DELETE){
+      $form->addElement(form_makeOpenTag('p'));
+      $form->addElement(form_makeCheckboxField('ow', 1, $lang['txt_overwrt'], 'dw__ow', 'check'));
+      $form->addElement(form_makeCloseTag('p'));
+    }
 
-        <p>
-          <label for="upload__name"><?php echo $lang['txt_filename']?>:</label>
-          <span class="nowrap">
-          <input type="text" name="id" class="edit" id="upload__name" /><input
-                 type="submit" class="button" value="<?php echo $lang['btn_upload']?>"
-                 accesskey="s" />
-          </span>
-        </p>
-
-        <?php if($auth >= AUTH_DELETE){?>
-            <p>
-              <input type="checkbox" name="ow" value="1" id="dw__ow" class="check" />
-              <label for="dw__ow" class="check"><?php echo $lang['txt_overwrt']?></label>
-            </p>
-        <?php }?>
-      </fieldset>
-    </form>
-    <?php
+    html_form('upload', $form);
 }
 
-
-
 /**
  * Build a tree outline of available media namespaces
  *
