@@ -11,7 +11,7 @@ if(!count($_POST) && $HTTP_RAW_POST_DATA){
   parse_str($HTTP_RAW_POST_DATA, $_POST);
 }
 
-if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
+if(!defined('DOKU_INC')) define('DOKU_INC',dirname(__FILE__).'/../../');
 require_once(DOKU_INC.'inc/init.php');
 require_once(DOKU_INC.'inc/common.php');
 require_once(DOKU_INC.'inc/pageutils.php');
@@ -28,7 +28,7 @@ if(isset($_POST['call']))
 else if(isset($_GET['call']))
   $call = 'ajax_'.$_GET['call'];
 else
-  return;
+  exit;
 if(function_exists($call)){
   $call();
 }else{
@@ -143,7 +143,7 @@ function ajax_lock(){
                   );
     $cname = getCacheName($draft['client'].$id,'.draft');
     if(io_saveFile($cname,serialize($draft))){
-      echo $lang['draftdate'].' '.date($conf['dformat']);
+      echo $lang['draftdate'].' '.strftime($conf['dformat']);
     }
   }
 
@@ -167,6 +167,8 @@ function ajax_draftdel(){
 
 /**
  * Return subnamespaces for the Mediamanager
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
  */
 function ajax_medians(){
   global $conf;
@@ -190,7 +192,9 @@ function ajax_medians(){
 }
 
 /**
- * Return subnamespaces for the Mediamanager
+ * Return list of files for the Mediamanager
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
  */
 function ajax_medialist(){
   global $conf;
@@ -199,5 +203,33 @@ function ajax_medialist(){
   media_filelist($_POST['ns']);
 }
 
+/**
+ * Return sub index for index view
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function ajax_index(){
+  global $conf;
+  require_once(DOKU_INC.'inc/search.php');
+  require_once(DOKU_INC.'inc/html.php');
+
+  // wanted namespace
+  $ns  = cleanID($_POST['idx']);
+  $dir  = utf8_encodeFN(str_replace(':','/',$ns));
+
+  $lvl = count(explode(':',$ns));
+
+  $data = array();
+  search($data,$conf['datadir'],'search_index',array('ns' => $ns),$dir);
+  foreach($data as $item){
+    $item['level'] = $lvl+1;
+    echo html_li_index($item);
+    echo '<div class="li">';
+    echo html_list_index($item);
+    echo '</div>';
+    echo '</li>';
+  }
+}
+
 //Setup VIM: ex: et ts=2 enc=utf-8 :
 ?>
