@@ -5,7 +5,7 @@
  * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
-if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
+if(!defined('DOKU_INC')) define('DOKU_INC',dirname(__FILE__).'/../../');
 define('DOKU_DISABLE_GZIP_OUTPUT',1);
 require_once(DOKU_INC.'inc/init.php');
 require_once(DOKU_INC.'inc/auth.php');
@@ -14,7 +14,7 @@ session_write_close();  //close session
 if(!defined('NL')) define('NL',"\n");
 
 // Version tag used to force rebuild on upgrade
-define('INDEXER_VERSION', 1);
+define('INDEXER_VERSION', 2);
 
 // keep running after browser closes connection
 @ignore_user_abort(true);
@@ -27,11 +27,21 @@ if(@ignore_user_abort() && !$conf['broken_iua']){
     $defer = true;
 }
 
+$ID = cleanID($_REQUEST['id']);
+
 // Catch any possible output (e.g. errors)
 if(!$_REQUEST['debug']) ob_start();
 
 // run one of the jobs
-runIndexer() or metaUpdate() or runSitemapper() or runTrimRecentChanges();
+$tmp = array();
+$evt = new Doku_Event('INDEXER_TASKS_RUN', $tmp);
+if ($evt->advise_before()) {
+  runIndexer() or 
+  metaUpdate() or 
+  runSitemapper() or 
+  runTrimRecentChanges() or 
+  $evt->advise_after();
+}
 if($defer) sendGIF();
 
 if(!$_REQUEST['debug']) ob_end_clean();
@@ -67,10 +77,10 @@ function runTrimRecentChanges() {
         !@file_exists($conf['changelog'].'_tmp')) {
             io_lock($conf['changelog']);
             $lines = file($conf['changelog']);
-            if (count($lines)<$conf['recent']) {
+            if (count($lines)<=$conf['recent']) {
                 // nothing to trim
                 io_unlock($conf['changelog']);
-                return true;
+                return false;
             }
 
             io_saveFile($conf['changelog'].'_tmp', '');          // presave tmp as 2nd lock
@@ -120,6 +130,7 @@ function runTrimRecentChanges() {
  * @author Andreas Gohr <andi@splitbrain.org>
  */
 function runIndexer(){
+    global $ID;
     global $conf;
     print "runIndexer(): started".NL;
 
@@ -135,7 +146,6 @@ function runIndexer(){
         trigger_event('TEMPORARY_INDEX_UPGRADE_EVENT', $tmp);
     }
 
-    $ID = cleanID($_REQUEST['id']);
     if(!$ID) return false;
 
     // check if indexing needed
@@ -167,6 +177,10 @@ function runIndexer(){
 
     require_once(DOKU_INC.'inc/indexer.php');
 
+    // upgrade to version 2
+    if (!@file_exists($conf['indexdir'].'/pageword.idx'))
+        idx_upgradePageWords();
+
     // do the work
     idx_addPage($ID);
 
@@ -184,9 +198,9 @@ function runIndexer(){
  * gain their data when viewed for the first time.
  */
 function metaUpdate(){
+    global $ID;
     print "metaUpdate(): started".NL;
 
-    $ID = cleanID($_REQUEST['id']);
     if(!$ID) return false;
     $file = metaFN($ID, '.meta');
     echo "meta file: $file".NL;
