@@ -6,7 +6,7 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
+if(!defined('DOKU_INC')) define('DOKU_INC',dirname(__FILE__).'/../../');
 if(!defined('NOSESSION')) define('NOSESSION',true); // we do not use a session or authentication here (better caching)
 if(!defined('NL')) define('NL',"\n");
 require_once(DOKU_INC.'inc/init.php');
@@ -45,13 +45,11 @@ function js_out(){
                 DOKU_INC.'lib/scripts/script.js',
                 DOKU_INC.'lib/scripts/tw-sack.js',
                 DOKU_INC.'lib/scripts/ajax.js',
+                DOKU_INC.'lib/scripts/index.js',
              );
     if($edit){
         if($write){
             $files[] = DOKU_INC.'lib/scripts/edit.js';
-            if($conf['spellchecker']){
-                $files[] = DOKU_INC.'lib/scripts/spellcheck.js';
-            }
         }
         $files[] = DOKU_INC.'lib/scripts/media.js';
     }
@@ -84,6 +82,9 @@ function js_out(){
     print "var notSavedYet = '".js_escape($lang['notsavedyet'])."';";
     print "var reallyDel   = '".js_escape($lang['del_confirm'])."';";
 
+    // load JS strings form plugins
+    $lang['js']['plugins'] = js_pluginstrings();
+    
     // load JS specific translations
     $json = new JSON();
     echo 'LANG = '.$json->encode($lang['js']).";\n";
@@ -114,17 +115,6 @@ function js_out(){
 
             // add lock timer
             js_runonstart("locktimer.init(".($conf['locktime'] - 60).",'".js_escape($lang['willexpire'])."',".$conf['usedraft'].")");
-
-            // load spell checker
-            if($conf['spellchecker']){
-                js_runonstart("ajax_spell.init('".
-                               js_escape($lang['spell_start'])."','".
-                               js_escape($lang['spell_stop'])."','".
-                               js_escape($lang['spell_wait'])."','".
-                               js_escape($lang['spell_noerr'])."','".
-                               js_escape($lang['spell_nosug'])."','".
-                               js_escape($lang['spell_change'])."')");
-            }
         }
     }
 
@@ -235,6 +225,34 @@ function js_pluginscripts(){
     return $list;
 }
 
+/**
+ * Return an two-dimensional array with strings from the language file of each plugin.
+ *
+ * - $lang['js'] must be an array. 
+ * - Nothing is returned for plugins without an entry for $lang['js']
+ *
+ * @author Gabriel Birke <birke@d-scribe.de>
+ */
+function js_pluginstrings()
+{
+    global $conf;
+    $pluginstrings = array();
+    $plugins = plugin_list();
+    foreach ($plugins as $p){
+        if (isset($lang)) unset($lang);
+        if (@file_exists(DOKU_PLUGIN."$p/lang/en/lang.php")) {
+            include DOKU_PLUGIN."$p/lang/en/lang.php";
+        }
+        if (isset($conf['lang']) && $conf['lang']!='en' && @file_exists(DOKU_PLUGIN."$p/lang/".$conf['lang']."/lang.php")) {
+            include DOKU_PLUGIN."$p/lang/".$conf['lang']."/lang.php";
+        }
+        if (isset($lang['js'])) {
+            $pluginstrings[$p] = $lang['js'];
+        }
+    }
+    return $pluginstrings;
+}
+
 /**
  * Escapes a String to be embedded in a JavaScript call, keeps \n
  * as newline
