@@ -181,7 +181,7 @@ media = {
             }
             return false;
         }
-        opener.insertTags('wiki__text','{{'+id+'|','}}','');
+        opener.insertTags('wiki__text','{{:'+id+'|','}}','');
 
         if(!media.keepopen) window.close();
         opener.focus();
